<?php

include_once 'includes/init.php';

function convert_server_to_GMT ( $offset = 0, $cutoffdate = '' ) {
  // Default value.
  $error = translate ( 'Conversion Successful' );
  // Don't allow $offsets over 24.
  if ( abs ( $offset ) > 24 )
    $offset = 0;
  // Do webcal_entry update.
  $res = dbi_execute ( 'SELECT cal_date, cal_time, cal_id, cal_duration
    FROM webcal_entry' );
  if ( $res ) {
    while ( $row = dbi_fetch_row ( $res ) ) {
      $cal_date = $row[0];
      $cal_time = sprintf ( "%06d", $row[1] );
      $cal_id = $row[2];
      $cal_duration = $row[3];
      // Skip Untimed or All Day events.
      if ( ( $cal_time == -1 ) || ( $cal_time == 0 && $cal_duration == 1440 ) )
        continue;
      else {
        $sy = substr ( $cal_date, 0, 4 );
        $sm = substr ( $cal_date, 4, 2 );
        $sd = substr ( $cal_date, 6, 2 );
        $sh = substr ( $cal_time, 0, 2 );
        $si = substr ( $cal_time, 2, 2 );
        $ss = substr ( $cal_time, 4, 2 );

        $new_datetime = ( empty ( $offset )
          ? mktime ( $sh, $si, $ss, $sm, $sd, $sy )
          : gmmktime ( $sh + $offset, $si, $ss, $sm, $sd, $sy ) );

        $new_cal_date = gmdate ( 'Ymd', $new_datetime );
        $new_cal_time = gmdate ( 'His', $new_datetime );
        $cutoff = ( ! empty ( $cutoffdate ) ? ' AND cal_date <= ?' : '' );
        // Now update row with new data.
        if ( ! dbi_execute ( 'UPDATE webcal_entry SET cal_date = ?, cal_time = ?
          WHERE cal_id = ?' . $cutoff,
            array ( $new_cal_date, $new_cal_time, $cal_id, $cutoffdate ) ) )
          return str_replace ( 'XXX', array ( 'webcal_entry', dbi_error () ),
            translate ( 'Error updating table XXX' ) );
      }
    }
    dbi_free_result ( $res );
  }

  // Do webcal_entry_logs update.
  $res = dbi_execute ( 'SELECT cal_date, cal_time, cal_log_id
    FROM webcal_entry_log' );
  if ( $res ) {
    while ( $row = dbi_fetch_row ( $res ) ) {
      $cal_date = $row[0];
      $cal_time = sprintf ( "%06d", $row[1] );
      $cal_log_id = $row[2];
      $sy = substr ( $cal_date, 0, 4 );
      $sm = substr ( $cal_date, 4, 2 );
      $sd = substr ( $cal_date, 6, 2 );
      $sh = substr ( $cal_time, 0, 2 );
      $si = substr ( $cal_time, 2, 2 );
      $ss = substr ( $cal_time, 4, 2 );
      $new_datetime = mktime ( $sh, $si, $ss, $sm, $sd, $sy );
      $new_cal_date = gmdate ( 'Ymd', $new_datetime );
      $new_cal_time = gmdate ( 'His', $new_datetime );
      // Now update row with new data
      if ( ! dbi_execute ( 'UPDATE webcal_entry_log
        SET cal_date = ?, cal_time = ? WHERE cal_log_id = ?',
          array ( $new_cal_date, $new_cal_time, $cal_log_id ) ) )
        return str_replace ( 'XXX', array ( 'webcal_entry_log', dbi_error () ),
          translate ( 'Error updating table XXX' ) );
    }
    dbi_free_result ( $res );
  }
  // Update Conversion Flag in webcal_config.
  // Delete any existing entry.
  if ( ! dbi_execute ( 'DELETE FROM webcal_config
    WHERE cal_setting = \'WEBCAL_TZ_CONVERSION\'' ) )
    return str_replace ( 'XXX', dbi_error (),
      translate ( 'Database error XXX.' ) );

  if ( ! dbi_execute ( 'INSERT INTO webcal_config ( cal_setting, cal_value )
    VALUES ( \'WEBCAL_TZ_CONVERSION\', \'Y\' )' ) )
    return str_replace ( 'XXX', dbi_error (),
      translate ( 'Database error XXX.' ) );

  return $error;
}

if (! $is_admin ) {
  echo "<p>You must be logged in as admin to do that!</p>";
} else {
  $tzoffset = getGetValue ('tzoffset');
  if (! empty ($tzoffset)) {
    do_config ('');
    $cutoffdate = getIntValue ('cutoffdate');
    $db_persistent = false;
    $db_type = $settings ['db_type'];
    $db_host = $settings ['db_host'];
    $db_database = $settings ['db_database'];
    $db_login = $settings ['db_login'];
    $db_password = $settings ['db_password'];
    $db_cachedir = getPostValue ('form_db_cachedir');
    // Avoid false visibilty of single user login
    $onload = 'auth_handler ();';
    $real_db = $db_database;
    if ($db_type == 'sqlite')
      $real_db = get_full_include_path ($db_database);
    if ($db_password == 'none')
      $db_password ='';
    $c = dbi_connect ($db_host, $db_login,
                      $db_password, $real_db, false);
    if ($c) {
      $ret = convert_server_to_GMT ($tzoffset, $cutoffdate);
      if (substr ($ret, 3, 21) == 'Conversion Successful') {
        echo "<p>Time zone successfully offset by $tzoffset hour(s)"
          . (! empty ($cutoffdate) ? ", starting on $cutoffdate" : "")
          . ".<p>";
        $response_msg = $tzSuccessStr;
      } else {
        $response_msg = translate ('Error Converting Timezone');
      }
    } else {
      $response_msg = $failure . dbi_error () . "</blockquote>\n";
    }
  } else {
    echo "<p>No time zone offset specified.</p>"
      . "<p>Usage example: http://this.server/webcalendar/change-tz=1</p>";
  }
}

?>
